# -*- python -*-
# $RCSfile: activeareamodmenu.py,v $
# $Revision: 1.20 $
# $Author: langer $
# $Date: 2004/06/15 18:22:53 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

from SWIG.common import switchboard
from common import activeareamod
from common import utils
from common.IO import automatic
from common.IO import mainmenu
from common.IO import oofmenu
from common.IO import parameter
from common.IO import whoville
import common.microstructure


def _undo(menuitem, microstructure):
    ms = common.microstructure.getMicrostructure(microstructure)
    activearea = ms.activearea
    activearea.begin_writing()
    activearea.undo()
    activearea.end_writing()
    switchboard.notify("active area modified", None)
    switchboard.notify("redraw")
    
def _redo(menuitem, microstructure):
    ms = common.microstructure.getMicrostructure(microstructure)
    activearea = ms.activearea
    activearea.begin_writing()
    activearea.redo()
    activearea.end_writing()
    switchboard.notify("active area modified", None)
    switchboard.notify("redraw")
                                                    
def _override(menuitem, microstructure, override):
    ms = common.microstructure.getMicrostructure(microstructure)
    ms.activearea.override(override)
    switchboard.notify("active area modified", None)
    switchboard.notify("redraw")

def activeAreaNameResolver(param, startname):
    if param.automatic():
        basename = 'activearea'
    else:
        basename = startname
    msname = param.group['microstructure'].value
    ms = common.microstructure.getMicrostructure(msname)
    return utils.uniqueName(basename, ms.activeAreaNames())

def _store(menuitem, microstructure, name):
    ms = common.microstructure.getMicrostructure(microstructure)
    ms.saveActiveArea(name)
    switchboard.notify('stored active areas changed', name)

def _rename(menuitem, microstructure, oldname, newname):
    ms = common.microstructure.getMicrostructure(microstructure)
    ms.renameActiveArea(oldname, newname)
    switchboard.notify('stored active areas changed', newname)

def _restore(menuitem, microstructure, name):
    ms = common.microstructure.getMicrostructure(microstructure)
    naa = ms.getNamedActiveArea(name)
    ms.activearea.begin_writing()
    naa.activearea.getTimeStamp().increment()
    ms.activearea.pushModification(naa.activearea)
    ms.activearea.end_writing()
    switchboard.notify('active area modified', None)
    switchboard.notify('redraw')

def _delete(menuitem, microstructure, name):
    ms = common.microstructure.getMicrostructure(microstructure)
    ms.deleteActiveArea(name)
    switchboard.notify('stored active areas changed', name)

# Build menu items for active area modifications

aamodmenu = mainmenu.OOF.addItem(oofmenu.OOFMenuItem('ActiveArea', cli_only=1))
                                                                                
def buildActiveAreaModMenu():
    aamodmenu.clearMenu()
                                                                                
    aamodmenu.addItem(
        oofmenu.OOFMenuItem('Undo', callback=_undo, params=[
            whoville.WhoParameter('microstructure',
                                  common.microstructure.microStructures)
            ]))
                                                                                
    aamodmenu.addItem(
        oofmenu.OOFMenuItem('Redo', callback=_redo, params=[
            whoville.WhoParameter('microstructure',
                                  common.microstructure.microStructures)
            ]))

    aamodmenu.addItem(
        oofmenu.OOFMenuItem('Override',
                            params=[
        parameter.BooleanParameter('override', 0),
        whoville.WhoParameter('microstructure',
                              common.microstructure.microStructures)
        ],
                            callback =_override))

    aamodmenu.addItem(
        oofmenu.OOFMenuItem('Store',
                            callback=_store,
                            params=parameter.ParameterGroup(
        whoville.WhoParameter('microstructure',
                              common.microstructure.microStructures),
        parameter.AutomaticNameParameter('name', value=automatic.automatic,
                                         resolver=activeAreaNameResolver))))

    aamodmenu.addItem(
        oofmenu.OOFMenuItem('Restore',
                            callback=_restore,
                            params=[
        whoville.WhoParameter('microstructure',
                              common.microstructure.microStructures),
        parameter.StringParameter('name')]))
    
    aamodmenu.addItem(
        oofmenu.OOFMenuItem('Rename',
                            callback=_rename,
                            params=[
        whoville.WhoParameter('microstructure',
                              common.microstructure.microStructures),
        parameter.StringParameter('oldname'),
        parameter.StringParameter('newname', '',
                                  tip='New name for the microstructure')
        ]))
    
    aamodmenu.addItem(
        oofmenu.OOFMenuItem('Delete',
                            callback=_delete,
                            params=[
        whoville.WhoParameter('microstructure',
                              common.microstructure.microStructures),
        parameter.StringParameter('name')]))
    
                           
                                                                                
    for registration in activeareamod.ActiveAreaModifier.registry:
        menuitem = aamodmenu.addItem(
            oofmenu.OOFMenuItem(utils.space2underscore(registration.name()),
                                callback=activeareamod.modify,
                                params=[
            whoville.WhoParameter('microstructure',
                                  common.microstructure.microStructures)
            ] + registration.params))
        menuitem.data = registration
                                                                                
buildActiveAreaModMenu()
