# -*- python -*-
# $RCSfile: menudump.py,v $
# $Revision: 1.7 $
# $Author: langer $
# $Date: 2004/08/06 20:54:33 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from common import enum
from common.IO import mainmenu
from common.IO import oofmenu
from common.IO import parameter
import string

indent =  "      "
indent2 = "           "

paramdict = {}                          # all parameter types that are used
regclassdict = {}                       # Every registered class we see.
enumdict = {}                           # All Enum types that are used

def dumper(menuitem, file):
    if menuitem.items:
        return                      # skip submenus, only do actual commands
    print >> file, menuitem.path()
    if menuitem.callback:
        try:
            # If the function has an "im_self", then it's a method,
            # and "im_self" is the class to which it is bound.
            im_self_data = menuitem.callback.im_self
        except AttributeError:
            funcname = ""
        else:
            funcname = im_self_data.__class__.__name__ + "."
        funcname += menuitem.callback.__name__
        print >> file, indent, "callback: Function '%s' in module '%s'" % \
              (funcname,
               menuitem.callback.func_globals['__name__'])
    if menuitem.helpstr:
        print >> file, indent, "help:", menuitem.helpstr
    else:
        print >> file, indent, "MISSING HELP STRING."
    if menuitem.secret:
        print >> file, indent, "secret:", menuitem.secret
    print >> file, indent, "threadability:", menuitem.threadable
    if menuitem.options:
        print >> file, indent, "options:",
        for key,val in menuitem.options.items():
            print >> file, key, "=", val,
        print >> file
    if menuitem.params:
        print >> file, indent, "Parameters:"
        for param in menuitem.params:
            print >> file, indent2, "'%s'\t%s" %(param.name, param.classRepr())
            add_params(param)
    print >> file
            # v = param.valueRepr()
            # if v:
            #     paramdict[param.classRepr()] = v


# Recursively add parameters amd registered classes to the dictionaries.
def add_params(param):
    global enumdict
    global paramdict
    global regclassdict
    v = param.valueRepr()
    if v:
        paramdict[param.classRepr()] = param
    # Check for RegisteredParameters
    try:
        rlist = param.registry
    except AttributeError:
        pass
    else:
        for r in rlist:
            regclassdict[r.subclass.__name__] = r
            for p in r.params:
                add_params(p)
    # Check for EnumParameters
    if isinstance(param, enum.EnumParameter):
        enumdict[param.enumclass] = 1


def menudump(menuitem, filename):
    global paramdict
    global enumdict
    paramdict = {}
    enumdict = {}
    file = open(filename, 'w')
    mainmenu.OOF.apply(dumper, file)
    
    print >> file, "\n\n------------ Registered Classes -------------\n"
    rlist = regclassdict.items()
    rlist.sort()
    for rname, reg in rlist:
        basenames = [regclass.__name__ for regclass in reg.registeredclasses]
        print >> file, indent, rname, "(%s)"%string.join(basenames,",")
        if reg.tip:
            print >> file, indent, "tip:", reg.tip
        else:
            print >> file, indent, "MISSING TIP STRING."
        for p in reg.params:
            print >> file, indent2, "'%s'\t%s" % (p.name, p.classRepr())
        print >> file

    print >> file, "\n\n------------ Enum Classes ------------\n"
    elist = enumdict.keys()
    elist.sort()
    for enum in elist:
        print >> file, indent, enum.__name__
        for name in enum.names:
            try:
                helpstr = enum.helpdict[name]
                print >> file, indent2, name, ": ", helpstr
            except KeyError:
                print >> file, indent2, name, ": MISSING HELP STRING"
        print >> file

    print >> file, "\n\n------------ Parameter Classes ------------\n"
    plist = paramdict.items()
    plist.sort()
    for name, p in plist:
        print >> file, indent, name #, v
        if p.tip:
            print >> file, indent2, "tip:", p.tip
        else:
            print >> file, indent2, "MISSING TIP STRING"
        vlist = string.split(p.valueRepr(), '\n')
        for vv in vlist:
            print >> file, indent2, vv
        print >> file
    file.close()

mainmenu.OOF.Help.addItem(oofmenu.OOFMenuItem(
    'API_Listing',
    callback=menudump,
    threadable = oofmenu.UNTHREADABLE,
    params=[parameter.StringParameter('filename', 'oof2_api.txt')],
    help="Create a verbose listing of all OOF menu commands."))
