# -*- python -*-
# $RCSfile: meshPage.py,v $
# $Revision: 1.66 $
# $Author: langer $
# $Date: 2004/09/22 20:46:01 $


# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from SWIG.common import switchboard
from SWIG.engine import masterelement
from common import debug
from common import guitop
from common import labeltree
from common import microstructure
from common.IO import mainmenu
from common.IO import parameter
from common.IO import reporter
from common.IO.GUI import chooser
from common.IO.GUI import oofGUI
from common.IO.GUI import parameterwidgets
from common.IO.GUI import whowidget
from engine import skeletoncontext
from gtk import *
import engine.mesh
import string

meshmenu = mainmenu.OOF.Mesh


class MeshPage(oofGUI.MainPage):
    def __init__(self):
        oofGUI.MainPage.__init__(self, name="FE Mesh", ordering=200,
                                 tip="Create a Finite Element Mesh from a Skeleton.")
        mainbox = GtkVBox(spacing=2)
        self.gtk.add(mainbox)

        hbox = GtkHBox()
        mainbox.add(hbox, expand=0, fill=0)
        centerbox = GtkHBox(spacing=3)
        hbox.add(centerbox, expand=1, fill=0)
        self.meshwidget = whowidget.WhoWidget(engine.mesh.meshes,
                                              callback=self.meshCB)
        label = GtkLabel("Microstructure=")
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        centerbox.add(self.meshwidget.gtk[0], expand=0, fill=0)

        label = GtkLabel("Skeleton=")
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        centerbox.add(self.meshwidget.gtk[1], expand=0, fill=0)

        label = GtkLabel("Mesh=")
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        centerbox.add(self.meshwidget.gtk[2], expand=0, fill=0)

        # Centered box of buttons
        hbox = GtkHBox()
        mainbox.add(hbox, expand=0, fill=0)
        bbox = GtkHBox(homogeneous=1, spacing=3)
        hbox.add(bbox, expand=1, fill=0)
        self.newbutton = GtkButton("New...")
        self.newbutton.connect('clicked', self.newCB)
        guitop.top().tooltip(self.newbutton, "Create a new mesh from the current skeleton.")
        bbox.add(self.newbutton, expand=1, fill=1)
        
        self.renamebutton = GtkButton("Rename...")
        self.renamebutton.connect('clicked', self.renameCB)
        guitop.top().tooltip(self.renamebutton, "Rename the current mesh.")   
        bbox.add(self.renamebutton, expand=1, fill=1)
        
        self.copybutton = GtkButton("Copy...")
        self.copybutton.connect('clicked', self.copyCB)
        guitop.top().tooltip(self.copybutton, "Copy the current mesh.")
        bbox.add(self.copybutton, expand=1, fill=1)
        
        self.deletebutton = GtkButton("Delete")
        self.deletebutton.connect('clicked', self.deleteCB)
        guitop.top().tooltip(self.deletebutton, "Delete the current mesh.")
        bbox.add(self.deletebutton, expand=1, fill=1)
        
        self.savebutton = GtkButton("Save...")
        self.savebutton.connect('clicked', self.saveCB)
        guitop.top().tooltip(self.savebutton, "Save the current mesh to a file.")
        bbox.add(self.savebutton, expand=1, fill=1)
        
        infoframe = GtkFrame('Mesh Information')
        scroll = GtkScrolledWindow()
        scroll.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        infoframe.add(scroll)
        self.infolist = GtkCList(cols=2)
        scroll.add(self.infolist)
        self.infolist.set_column_auto_resize(0, 1)
        self.infolist.set_column_auto_resize(1, 1)

        mainbox.add(infoframe)

        switchboard.requestCallback(("new who", "Skeleton"), self.newSkeleton)
        switchboard.requestCallback(("new who", "Mesh"), self.newMesh)
        switchboard.requestCallback(("remove who", "Mesh"), self.removeMesh)

        self.sensitize()

    def show(self):
        debug.mainthreadTest()
        self.gtk.show_all()
        self.update()

    #######################

    def currentMSName(self):
        path = labeltree.makePath(self.currentFullMeshName())
        return path[0]
    def currentMS(self):
        msname = self.currentMSName()
        if msname:
            return microstructure.microStructures[msname].getObject()

    def currentSkeletonPath(self):
        path = labeltree.makePath(self.currentFullMeshName())
        return path[0:2]
    def currentSkeletonFullName(self):
        return string.join(self.currentSkeletonPath(), ":")
    def currentSkeleton(self):
        ms = self.currentMS()
        skelname = self.currentSkeletonPath()[-1]
        if skelname:
            return skeletoncontext.getSkeleton(ms, skelname)

    def currentFullMeshName(self):
        return self.meshwidget.get_value()
    def currentMeshName(self):
        path = labeltree.makePath(self.currentFullMeshName())
        return path[2]
    def currentMesh(self):
        try:
            return engine.mesh.meshes[self.currentFullMeshName()].getObject()
        except KeyError:
            return None
            
    def sensitize(self):
        debug.mainthreadTest()
        msok = self.currentMS() is not None
        skelok = msok and self.currentSkeleton() is not None
        meshok = skelok and self.currentMesh() is not None
        self.newbutton.set_sensitive(skelok)
        self.deletebutton.set_sensitive(meshok)
        self.renamebutton.set_sensitive(meshok)
        self.copybutton.set_sensitive(meshok)
        self.savebutton.set_sensitive(meshok)
        
    def update(self):
        self.set_state(self.currentFullMeshName())

    def set_state(self, meshpath):
        debug.mainthreadTest()
        path = labeltree.makePath(meshpath)
        self.meshwidget.set_value(path)
        if path[-1]:                    # ie, mesh name is not empty
            themesh = engine.mesh.meshes[path]
            skel = themesh.skeleton

            self.infolist.clear()
            row = 0
            masterelementenums = masterelement.getMasterElementEnums()
            sides = masterelementenums.keys() # list of element geometries
            sides.sort()
            for nsides in sides:
                self.infolist.append([`nsides` +" sided element:",
                                      themesh.getMasterElementType(nsides)])
                self.infolist.set_selectable(row, 0)
                row += 1
            
        else:                           # no current mesh
            self.infolist.clear()
            self.infolist.append(['No mesh!', ''])
            self.infolist.set_selectable(0, 0)


    def newSkeleton(self, skeletonname): # switchboard ("new who", "Skeleton")
        self.sensitize()

    def newMesh(self, meshname):        # switchboard ("new who", "Mesh")
        self.set_state(meshname)
        self.sensitize()
        
    def meshCB(self, meshpath):         # WhoWidget callback
        self.set_state(meshpath)
        self.sensitize()

    def newCB(self, *args):             # gtk button callback
        menuitem = mainmenu.OOF.Mesh.New
        params = filter(lambda x: x.name !='skeleton', menuitem.params)
        if parameterwidgets.getParameters(title='Create a new mesh', *params):
            menuitem.callWithDefaults(skeleton=self.currentSkeletonFullName())

    def deleteCB(self, *args):          # gtk button callback
        if reporter.query(
            "Really delete %s?"%self.currentFullMeshName(),
            "No", default="Yes") == "Yes":
            meshmenu.Delete(mesh=self.currentFullMeshName())

    def removeMesh(self, path):    # switchboard ("remove who", "Mesh")
        self.update()
        self.sensitize()

    def copyCB(self, *args):            # gtk button callback
        menuitem = mainmenu.OOF.Mesh.Copy
        nameparam = menuitem.get_arg("name")
        fieldparam = menuitem.get_arg("copy_field")
        eqnparam = menuitem.get_arg("copy_equation")
        bcparam = menuitem.get_arg("copy_bc")
        if parameterwidgets.getParameters(nameparam, fieldparam, eqnparam,
                                          bcparam, title='Copy a mesh'):
            menuitem.callWithDefaults(mesh=self.currentFullMeshName())
        
    def renameCB(self, *args):          # gtk button callback
        menuitem = meshmenu.Rename
        namearg = menuitem.get_arg('name')
        curmeshpath = self.currentFullMeshName()
        namearg.value = labeltree.makePath(curmeshpath)[-1]
        if parameterwidgets.getParameters(namearg,
                                          title='Rename mesh '+namearg.value):
            menuitem.callWithDefaults(mesh=curmeshpath)

    def saveCB(self, *args):
        menuitem = mainmenu.OOF.File.Save.Mesh
        meshname = self.meshwidget.get_value()
        file = fileselector.fileSelector(
            ident='SaveMeshFromPage',
            title='Save Mesh "%s"' % meshname,
            params=[menuitem.get_arg('format')])
        if file is not None:
            mode = fileselector.get_file_mode(file)
            if mode is not None:
                menuitem.callWithDefaults(filename=file, mode=mode,
                                          mesh=meshname)

#############
        
mp = MeshPage()

###########################################################

from common.IO.GUI import fileselector

def _save_mesh(menuitem):
    if len(engine.mesh.meshes) > 0:
        file = fileselector.fileSelector(
            ident='SaveMeshFromMenu',
            title='Save Mesh',
            params=[menuitem.get_arg('mesh'), menuitem.get_arg('format')]
            )
        if not file is None:
            mode = fileselector.get_file_mode(file)
            if mode is not None:
                menuitem.callWithDefaults(filename=file, mode=mode)
    else:
        reporter.report("No mesh?")

mainmenu.OOF.File.Save.Mesh.gui_callback = _save_mesh
