# -*- python -*-
# $RCSfile: movenodeGUI.py,v $
# $Revision: 1.44 $
# $Author: langer $
# $Date: 2004/09/23 15:55:23 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from SWIG.common.IO.GUI import rubberband
from SWIG.common import switchboard
from common import debug
from common import guitop
from common import primitives
from common.IO import mainmenu
from common.IO.GUI import mousehandler
from common.IO.GUI import toolboxGUI
from engine.IO import movenode
from gtk import *

class MoveNodeToolboxGUI(toolboxGUI.GfxToolbox, mousehandler.MouseHandler):
    def __init__(self, movenodetoolbox):
        debug.mainthreadTest()
        self.downed = 0                 # is the mouse button down?
        self.moving = 0                 # in the middle of a move
        self.node = None                # node being moved
        self.writable = 1               # is the top most Skeleton writable?
        self.mode = "Mouse"
        
        toolboxGUI.GfxToolbox.__init__(self, "Move Nodes", movenodetoolbox)
        mainbox = GtkVBox()
        self.gtk.add(mainbox)

        hbox = GtkHBox()
        mainbox.pack_start(hbox, expand =0, fill=0)
        hbox.pack_start(GtkLabel("Move with: "), expand=0, fill=0)

        modes = ["Mouse", "Keyboard"]
        self.modebuttons = []
        for mode in modes:
            if self.modebuttons:
                button = GtkRadioButton(label=mode,
                                        group=self.modebuttons[0])
            else:
                button = GtkRadioButton(label=mode)
            self.modebuttons.append(button)
            if mode=="Mouse":
                guitop.top().tooltip(button,
                                     "Click and drag a node to move it.")
            elif mode=="Keyboard":
                guitop.top().tooltip(button,
                                     "Select a node/Input a destination and click \
Move button.")
            hbox.pack_start(button, expand=0, fill=0)
            button.set_active(self.mode is mode)
            button.connect('clicked', self.changeMode, mode)

        # allow illegal move?
        self.allow_illegal = GtkCheckButton("Allow illegal moves")
        mainbox.pack_start(self.allow_illegal, expand=0, fill=0, padding=1)
        mainbox.pack_start(GtkHSeparator(), expand=0, fill=0, padding=3)

        self.table = GtkTable(cols=2, rows=6)
        mainbox.pack_start(self.table, expand=0, fill=0)

        label = GtkLabel('x=')
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 0,1, 0,1, xpadding=5, xoptions=FILL)
        self.xtext = GtkEntry()
        self.xtext.set_editable(1)
        self.table.attach(self.xtext, 1,2, 0,1,
                          xpadding=5, xoptions=EXPAND|FILL)
        guitop.top().tooltips.set_tip(self.xtext, "x position of the mouse")

        label = GtkLabel('y=')
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 0,1, 1,2, xpadding=5, xoptions=FILL)
        self.ytext = GtkEntry()
        self.ytext.set_editable(1)
        self.table.attach(self.ytext, 1,2, 1,2,
                          xpadding=5, xoptions=EXPAND|FILL)
        guitop.top().tooltips.set_tip(self.xtext, "y position of the mouse")
        
        label = GtkLabel("Change in... ")
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 0,1, 2,3, xpadding=4, xoptions=EXPAND|FILL)

        label = GtkLabel("shape energy=")
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 0,1, 3,4, xpadding=5, xoptions=FILL)
        self.shapetext = GtkEntry()
        self.shapetext.set_editable(0)
        self.table.attach(self.shapetext, 1,2, 3,4,
                          xpadding=5, xoptions=EXPAND|FILL)
        guitop.top().tooltips.set_tip(self.shapetext,
                                      "total change in shape energy")

        label = GtkLabel("homogeneity=")
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 0,1, 4,5, xpadding=5, xoptions=FILL)
        self.homogtext = GtkEntry()
        self.homogtext.set_editable(0)
        self.table.attach(self.homogtext, 1,2, 4,5,
                          xpadding=5, xoptions=EXPAND|FILL)
        guitop.top().tooltips.set_tip(self.homogtext,
                                      "total change in homogeneity")

        mainbox.pack_start(GtkHSeparator(), expand=0, fill=0, padding=3)
        self.statusbox = GtkVBox()
        mainbox.pack_start(self.statusbox, expand=0, fill=0, padding=3)
        
        bbox = GtkHButtonBox()
        mainbox.pack_end(bbox, expand=0, fill=0, padding=3)
        self.undobutton = GtkButton('Undo')
        guitop.top().tooltip(self.undobutton, "Undo the latest node move.")
        self.movebutton = GtkButton('Move')
        guitop.top().tooltip(self.movebutton, "Move the selected node to the specified position.")
        self.redobutton = GtkButton('Redo')
        guitop.top().tooltip(self.redobutton, "Redo the latest UNDO.")
        
        bbox.pack_start(self.undobutton, expand=0, fill=0, padding=5)
        bbox.pack_start(self.movebutton, expand=0, fill=0, padding=5)
        bbox.pack_end(self.redobutton, expand=0, fill=0, padding=5)
        self.undobutton.connect('clicked', self.undoCB)
        self.movebutton.connect('clicked', self.moveCB)
        self.redobutton.connect('clicked', self.redoCB)

        self.sbcallbacks = [
            switchboard.requestCallback(('who changed', 'Skeleton'),
                                        self.skelChanged),
            switchboard.requestCallback("made reservation", self.rsrvChanged,
                                        1),
            switchboard.requestCallback("cancelled reservation",
                                        self.rsrvChanged, 0)
            ]

    def changeMode(self, button, mode):
        self.mode = mode
        self.toolbox.selectednode.set()
        self.node = None
        self.sensitize()
        switchboard.notify('redraw')
        
    def getSkeletonContext(self):
        return self.gfxwindow().topwho('Skeleton')
    
    def getSkeleton(self):
        return self.gfxwindow().topmost('Skeleton')

    def updateStatus(self):
        nillegal = self.getSkeleton().nillegal()
        if nillegal:
            text = "%d illegal element%s in the skeleton.\n" % \
                   (nillegal, "s"*(nillegal!=1))
        else:
            text = ""
        label=GtkLabel(text)
        self.statusbox.foreach(GtkObject.destroy)
        self.statusbox.add(label)
        label.show()        

    def skelChanged(self, skelcontext):
        if skelcontext is self.getSkeletonContext():
            self.updateStatus()  # no. of illegal elements
            # node selected for moving (in Keyboard mode) but not yet moved.
            if self.mode == "Keyboard" and self.node:
                self.toolbox.selectednode.set()
                self.node = None
            self.sensitize()
        
    def sensitize(self):
        debug.mainthreadTest()
        skeletoncontext = self.getSkeletonContext()
        reserved = skeletoncontext.query_reservation()
        self.undobutton.set_sensitive(skeletoncontext is not None
                                      and not reserved
                                      and skeletoncontext.undoable())
        self.redobutton.set_sensitive(skeletoncontext is not None
                                      and not reserved
                                      and skeletoncontext.redoable())
        
        self.movebutton.set_sensitive(self.mode == "Keyboard"
                                      and self.node is not None
                                      and not reserved)

    def rsrvChanged(self, reserved, whocls):
        skelcontext = self.getSkeletonContext()
        if skelcontext and skelcontext.path() == whocls.path():
            self.sensitize()
            self.writable = not reserved
            debug.fmsg('self.writable =', self.writable)

    def activate(self):
        toolboxGUI.GfxToolbox.activate(self)
        self.gfxwindow().setMouseHandler(self)
        self.sensitize()

    def deactivate(self):
        self.node = None
        toolboxGUI.GfxToolbox.deactivate(self)
        self.gfxwindow().removeMouseHandler()

    def close(self):
        map(switchboard.removeCallback, self.sbcallbacks)
        toolboxGUI.GfxToolbox.close(self)

    def showPosition(self, x, y):
        debug.mainthreadTest()
        self.xtext.set_text("%-11.4g" % x)
        self.ytext.set_text("%-11.4g" % y)

    def acceptEvent(self, eventtype):
        if not self.writable: # Don't accept any events if the skeleton is busy.
            return 0       
        if self.mode == "Mouse":
            return eventtype=='down' or \
                   (self.downed and eventtype in ('move', 'up'))
        elif self.mode == "Keyboard":
            return eventtype=='up'
        
    def down(self, x, y, shift, ctrl):
        self.downed = 1
        self.showPosition(x, y)
        skeleton = self.getSkeleton()
        reserved = self.getSkeletonContext().query_reservation()
        if skeleton is not None and not reserved:
            self.node = skeleton.nearestNode(primitives.Point(x,y))
            if self.node.pinned():
                self.node = None
            else:
                # Mouse-down events are only accepted in Mouse mode,
                # so the end_reading() call corresponding to this
                # begin_reading() is in the "Mouse" section of the
                # up() callback.
                self.getSkeletonContext().begin_reading()
                self.nbrnodes = self.node.neighborNodes(skeleton)
                self.nbrelements = self.node.neighborElements()
                self.startpt = self.node.position()
                # Get initial values of homogeneity and shape energy
                self.homogeneity0 = 0.0
                self.shapeenergy0 = 0.0
                for element in self.nbrelements:
                    if not element.illegal():
                        self.homogeneity0 += element.homogeneity(skeleton)
                        self.shapeenergy0 += element.energyShape()
                # Create rubberband
                points = [n.position() for n in self.nbrnodes]
                self.gfxwindow().setRubberband(
                    rubberband.SpiderRubberBand(points))
            
    def move(self, x, y, shift, ctrl):
        debug.mainthreadTest()
        if self.node is not None:
            self.moving = 1
            self.showPosition(x, y)
            homogeneity = 0.0
            shapeenergy = 0.0
            skeleton = self.getSkeleton()
            # It's generally forbidden to call node.moveTo instead of
            # skeleton.moveNodeTo, but since we're going to move the
            # node back (see up(), below), it's ok.
            self.node.moveTo(primitives.Point(x,y))
            if self.node.illegal():
                self.homogtext.set_text("---")
                self.shapetext.set_text("---")
                label=GtkLabel("Illegal node position!")
            else:
                for element in self.nbrelements:
                    homogeneity += element.homogeneity(skeleton)
                    shapeenergy += element.energyShape()
                self.homogtext.set_text("%-11.4g" %
                                        (homogeneity-self.homogeneity0))
                self.shapetext.set_text("%-11.4g" %
                                        (shapeenergy-self.shapeenergy0))
                label = GtkLabel("")
            self.statusbox.foreach(GtkObject.destroy)
            self.statusbox.add(label)
            label.show()
        
    def up(self, x, y, shift, ctrl):
        if self.mode == "Mouse":
            self.getSkeletonContext().end_reading()
            if self.moving:
                self.moving = 0 # Dunmovin, CA is just east of the Sierra Nevada
                self.showPosition(x, y)
                # Accept/reject conditon:
                # - Allow_illegal is ON : unconditionally accept everything
                # - Allow_illegal is OFF : any node asociated with illegal node
                # before or after will not be moved.

                if self.node.illegal() and not self.allow_illegal.get_active():
                    # Restore the state of the skeleton.
                    self.node.moveTo(self.startpt)
                    self.updateStatus()
                else:
                    # Restore the state of the skeleton so that the non-GUI
                    # movenode command can do the real work.
                    self.node.moveTo(self.startpt)
                    try:
                        self.toolbox.menu.MoveNode(
                            origin=self.node.position(),
                            destination=primitives.Point(x,y))
                    finally:
                        self.gfxwindow().setRubberband(
                            rubberband.NoRubberBand())
                        self.node = None
                        self.nbrnodes = []
            self.downed = 0
            self.node = None
                
        elif self.mode == "Keyboard":
            if not self.node:
                self.toolbox.menu.SelectNode(x=x, y=y)
                self.node = self.toolbox.selectednode.node()
                self.showPosition(self.node.position().x,
                                  self.node.position().y)
                self.sensitize()
            else:
                self.showPosition(x, y)

    def undoCB(self, button):
        mainmenu.OOF.Skeleton.Undo(skeleton=self.getSkeletonContext().path())
        if self.mode=="Mouse":
            self.sensitize()
        else:
            self.node = None
            self.sensitize()

    # Move button callback in Keyboard mode
    def moveCB(self, button):
        debug.mainthreadTest()
        x = float(self.xtext.get_text())
        y = float(self.ytext.get_text())
        somewhere = primitives.Point(x, y)

        # Computing initial energy of neighboring elements
        skeleton = self.getSkeleton()
        neighbors = self.node.neighborElements()

        homogeneity0 = 0.0
        shapeenergy0 = 0.0
        for element in neighbors:
            if element.illegal():
                continue
            homogeneity0 += element.homogeneity(skeleton)
            shapeenergy0 += element.energyShape()

        # Energy after node move
        self.node.moveTo(somewhere)
        illegal = 0
        homogeneity = 0.0
        shapeenergy = 0.0
        for element in neighbors:
            if not illegal:
                if element.illegal():
                    illegal = 1
                else:
                    homogeneity += element.homogeneity(skeleton)
                    shapeenergy += element.energyShape()
        self.node.moveBack()

        if illegal:
            if self.allow_illegal.get_active():
                self.toolbox.menu.MoveNode(origin=self.node.position(),
                                           destination=somewhere)
            else:
                self.updateStatus()
            self.homogtext.set_text("---")
            self.shapetext.set_text("---")
        else:
            self.toolbox.menu.MoveNode(origin=self.node.position(),
                                       destination=somewhere)
            self.homogtext.set_text("%-11.4g" % (homogeneity - homogeneity0))
            self.shapetext.set_text("%-11.4g" % (shapeenergy - shapeenergy0))
            
        self.toolbox.selectednode.set()
        self.node = None
        self.sensitize()
        switchboard.notify('redraw')

    def redoCB(self, button):
        mainmenu.OOF.Skeleton.Redo(skeleton=self.getSkeletonContext().path())
        if self.mode=="Mouse":
            self.sensitize()
        else:
            self.node = None
            self.sensitize()
            
def _makeGUI(self):
    return MoveNodeToolboxGUI(self)

movenode.MoveNodeToolbox.makeGUI = _makeGUI
