# -*- python -*-
# $RCSfile: profilefunctionwidget.py,v $
# $Revision: 1.8 $
# $Author: reida $
# $Date: 2004/07/12 15:28:09 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

import gtk
import re, string
from SWIG.common import ooferror
from SWIG.common import switchboard
from common.IO.GUI import parameterwidgets
from common.IO.GUI import regclassfactory
from engine import profilefunction
from engine import profile
from engine.IO.GUI import meshparamwidgets


class ProfileFunctionWidget(parameterwidgets.XYStrFunctionWidget):
    def get_value(self):
        return profilefunction.ProfileFunction(self.gtk.get_text())

def PFP_makeWidget(self, scope):
    return ProfileFunctionWidget(self, scope)

profilefunction.ProfileFunctionParameter.makeWidget = PFP_makeWidget
    

# Local utility class -- keeps track of a GtkHBox, and the contained
# GtkLabel and Registered Class Factory.  Allows the label to be
# dynamically reset.  Exposes a small part of the enclosed Registered
# Class Factory interface.
class LabelledProfileRCF:
    def __init__(self, scope=None, label=""):
        self.gtk = gtk.GtkHBox()
        self.label = gtk.GtkLabel(label+" = ")
        self.widget = regclassfactory.RegisteredClassFactory(
            profile.Profile.registry, scope=scope)
        self.gtk.add(self.label)
        self.gtk.add(self.widget.gtk)
        self.show()
    def show(self):
        self.label.show()
        self.widget.show()
        self.gtk.show()
    def set_label(self, label):
        self.label.set_text(label)
        self.show()
    def get_value(self):
        return self.widget.get_value()
    def set(self, value):
        self.widget.set(value)
        
    
# Widget for FluxProfileSet objects -- this widget goes with the
# FluxProfileSetParameter, and consequently only comes up in contexts
# where a flux can be retrieved -- it is a container for that number
# of profile widgets.
class FluxProfileSetWidget(parameterwidgets.ParameterWidget):
    def __init__(self, param, scope=None):
        if scope==None:
            raise ooferror.ErrPyProgrammingError(
                "FluxProfileSetWidget instanced with no scope.")
        self.scope=scope
        self.fluxwidget = scope.findWidget(
            lambda x: x.__class__==meshparamwidgets.FluxParameterWidget)
        self.sbcb = switchboard.requestCallback(
            self.fluxwidget, self.checkFlux)
        gtk_obj = gtk.GtkVBox()
        parameterwidgets.ParameterWidget.__init__(self, gtk_obj, scope)
        self.profileset = [] # Profiles themselves.
        self.widgetset = []  # RegisteredClassFactories for profiles.
        self.set_value(param.value)
        # TODO LATER: The real right way to handle validity is to behave
        # as a proper container, and catch the validity changes of the
        # enclosed RCFs.  However, we happen to know that we are always
        # valid, so we do it the simple way.
        self.setValidity(1)

    # Changes the length of the list of profile sets, padding with
    # ConstantProfile(0.0) as required to get the length.
    def checkFlux(self):
        flux = self.fluxwidget.get_value()
        self.nslots = flux.divergence_dim()
        if len(self.profileset) > self.nslots:
            self.profileset = self.profileset[:self.nslots]
        elif len(self.profileset) < self.nslots:
            self.profileset += [profile.ConstantProfile(0.0)]*\
                               (self.nslots-len(self.profileset))
        self.show()
            
    # Parses the funcstr.
    def set_value(self, newvalue):
        if newvalue:
            self.profileset = newvalue.get_profiles()
        self.checkFlux()

    def get_value(self):
        return profile.FluxProfileSet([w.get_value() for w in self.widgetset])
            

    # Draw the right number of slots with the right widgets in them.
    # The widgets are LabelledProfileRCF objects, see above.
    def show(self):
        if len(self.widgetset) > self.nslots:
            for w in self.widgetset[self.nslots:]:
                self.gtk.remove(w.gtk)
            self.widgetset = self.widgetset[:self.nslots]
        elif len(self.widgetset) < self.nslots:
            for i in range(len(self.widgetset), self.nslots):
                new_widget = LabelledProfileRCF(scope=self.scope,
                                                label="p%d" % i)
                # new_widget = regclassfactory.RegisteredClassFactory(
                #     profile.Profile.registry, scope=self.scope)
                self.widgetset.append(new_widget)
                self.gtk.add(new_widget.gtk)
        # Widget list and profile set are now the same length. 
        for i in range(len(self.widgetset)):
            self.widgetset[i].set(self.profileset[i])
            self.widgetset[i].show()
        self.gtk.show()


    def __del__(self):
        switchboard.removeCallback(self.sbcb)
        

def FPSP_makeWidget(self, scope):
    return FluxProfileSetWidget(self, scope)

profile.FluxProfileSetParameter.makeWidget = FPSP_makeWidget
