# -*- python -*-
# $RCSfile: pinnodemodifier.py,v $
# $Revision: 1.5 $
# $Author: haan $
# $Date: 2004/02/12 22:19:16 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

from common import registeredclass
from common.IO import parameter
from common import primitives
from engine import skeletoncontext
from engine.IO import skeletongroupparams
import random

def pinnodemod(menuitem, context, **params):
    registration = menuitem.data
    modifier = apply(registration, (), params)
    modifier(context)

class PinNodeModifier(registeredclass.RegisteredClass):
    registry = []


class PinNodeSelection(PinNodeModifier):
    def __call__(self, context):
        skelcontext = skeletoncontext.skeletonContexts[context]
        pinnednodes = skelcontext.pinnednodes
        pinnednodes.start()
        pinnednodes.pin(skelcontext.nodeselection.retrieve())
        pinnednodes.signal()

registeredclass.Registration('Pin Node Selection',
                             PinNodeModifier,
                             PinNodeSelection, ordering=0,
                             tip="Pin selected nodes.")


class UnPinNodeSelection(PinNodeModifier):
    def __call__(self, context):
        skelcontext = skeletoncontext.skeletonContexts[context]
        pinnednodes = skelcontext.pinnednodes
        pinnednodes.start()
        pinnednodes.unpin(skelcontext.nodeselection.retrieve())
        pinnednodes.signal()

registeredclass.Registration('UnPin Node Selection',
                             PinNodeModifier,
                             UnPinNodeSelection, ordering=1,
                             tip="Unpin selected nodes if pinned.")


class PinInternalBoundaryNodes(PinNodeModifier):
##     def __init__(self, skel_bound=1):
##         self.skel_bound = skel_bound
    
##     def getCategories(self, context):
##         ms = context.getObject().MS
##         ncats = ms.nCategories()
##         cats = []
##         x = range(0, ms.sizeInPixels()[0])
##         random.shuffle(x)
##         y = range(0, ms.sizeInPixels()[1])
##         random.shuffle(y)
##         for i in x:
##             for j in y:
##                 cat = ms.categoryFromPoint(primitives.iPoint(i,j))
##                 if cat not in cats:
##                     cats.append(cat)
##                     if len(cats) == ncats:
##                         return cats

##     def getElementSets(self, context, cats):
##         elsets = {}
##         for cat in cats:
##             elsets[cat] = []
##         skeleton = context.getObject()
##         for element in skeleton.element_iterator():
##             elsets[element.dominantPixel(skeleton)].append(element)
##         return elsets

##     def getBoundaryNodes(self, context, elset):
##         skel = context.getObject()
##         for element in elset:
##             for i in range(element.nnodes()):
##                 n0 = element.nodes[i]
##                 n1 = element.nodes[(i+1)%element.nnodes()]
##                 seg = skel.getSegment(n0, n1)
##                 # A segment is on the boundary of the selection if it
##                 # belongs to only one element.
##                 n = 0
##                 for el in seg.getElements():
##                     if el in elset:
##                         n += 1
##                 if n == 1:
##                     self.addNode(n0)
##                     self.addNode(n1)      

##     def addNode(self, node):
##         try:
##             self.boundNodes[node] += 1
##         except KeyError:
##             self.boundNodes[node] = 1

##     def removeBoundNodes(self):
##         for node in self.boundNodes.keys():
##             if not (node.movable_x() and node.movable_y()):
##                 # It's a node on skeleton boundary.
##                 # BUT, a node at a junction where a different element group
##                 # intersects better be included.
##                 if self.boundNodes[node]==2:
##                     del self.boundNodes[node]

    def __call__(self, skeleton):
        context = skeletoncontext.skeletonContexts[skeleton]
        skel = context.getObject()
        pinnednodes = context.pinnednodes
        nodelist = []
        for node in skel.nodes:
            elements = node.neighborElements()
            cat = elements[0].dominantPixel(skel)
            for element in elements[1:]:
                if cat != element.dominantPixel(skel):
                    nodelist.append(node)
                    break
        pinnednodes.start()
        pinnednodes.pin(nodelist)
        pinnednodes.signal()

registeredclass.Registration('Pin Internal Boundary Nodes',
                             PinNodeModifier,
                             PinInternalBoundaryNodes, ordering=2,
                             tip="Pin all material-boundary nodes.")


class PinSelectedSegments(PinNodeModifier):
    def __call__(self, context):
        skelcontext = skeletoncontext.skeletonContexts[context]
        pinnednodes = skelcontext.pinnednodes
        nodes = {}
        for segment in skelcontext.segmentselection.retrieve():
            nodes[segment.nodes()[0]] = None
            nodes[segment.nodes()[1]] = None
        pinnednodes.start()
        pinnednodes.pin(nodes.keys())
        pinnednodes.signal()

registeredclass.Registration('Pin Selected Segments',
                             PinNodeModifier,
                             PinSelectedSegments, ordering=4,
                             tip="Pin nodes of selected segments.")


class PinSelectedElements(PinNodeModifier):
    def __init__(self, internal=0, boundary=1):
        self.internal = internal
        self.boundary = boundary

    def getAllNodes(self, context):
        nodes = {}
        for element in context.elementselection.retrieve():
            for nd in element.nodes:
                nodes[nd] = None
        return nodes.keys()

    def getBoundaryNodes(self, context):
        bound = {}
        for element in context.elementselection.retrieve():
            for i in range(element.nnodes()):
                n0 = element.nodes[i]
                n1 = element.nodes[(i+1)%element.nnodes()]
                seg = context.getObject().getSegment(n0, n1)
                # A segment is on the boundary of the selection if it
                # belongs to only one element.
                n = 0
                for el in seg.getElements():
                    if el.selected: n += 1
                if n == 1:
                    bound[n0] = None
                    bound[n1] = None
        return bound.keys()

    def getInternalNodes(self, context, allnodes):
        bound = self.getBoundaryNodes(context)
        internal = []
        for nd in allnodes:
            if nd not in bound:
                internal.append(nd)
        return internal

    def __call__(self, context):
        skelcontext = skeletoncontext.skeletonContexts[context]
        pinnednodes = skelcontext.pinnednodes
        if self.internal and self.boundary:
            nodes = self.getAllNodes(skelcontext)
        elif not self.internal and self.boundary:
            nodes = self.getBoundaryNodes(skelcontext)
        elif self.internal and not self.boundary:
            allnodes = self.getAllNodes(skelcontext)
            nodes = self.getInternalNodes(skelcontext, allnodes)
        else:
            nodes = []
        pinnednodes.start()
        pinnednodes.pin(nodes)
        pinnednodes.signal()

registeredclass.Registration('Pin Selected Elements',
                             PinNodeModifier,
                             PinSelectedElements, ordering=5,
                             params=[
    parameter.BooleanParameter('internal', 0, tip='Select internal nodes.'),
    parameter.BooleanParameter('boundary', 1, tip='Select boundary nodes.')],
                             tip="Pin nodes of selected elements.")
