# -*- python -*-
# $RCSfile: pinnodesdisplay.py,v $
# $Revision: 1.6 $
# $Author: langer $
# $Date: 2004/09/16 19:14:18 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

from common import color
from common import debug
from common import registeredclass
from common.IO import display
from common.IO import parameter

class PinnedNodesDisplay(display.DisplayMethod):
    def __init__(self, color, size):
        self.color = color
        self.size = size
        display.DisplayMethod.__init__(self)
    def draw(self, gfxwindow, device):
        skel = self.who().resolve(gfxwindow)
        device.set_lineColor(self.color)
        device.set_lineWidth(self.size)
        for node in skel.pinnednodes.retrieve():
            device.draw_dot(node.position())
    def getTimeStamp(self, gfxwindow):
        return max(self.timestamp,
                   self.who().resolve(gfxwindow).pinnednodes.timestamp)

pinnedNodesDisplay = registeredclass.Registration(
    'Pinned Nodes',
    display.DisplayMethod,
    PinnedNodesDisplay,
    params=[color.ColorParameter('color', color.RGBColor(0.93, 0.93, 0.0)),
            parameter.IntRangeParameter('size', (0,10), 2)],
    layerordering=display.PointLike(2),
    ordering=3.1,
    whoclasses=('Skeleton',),
    tip="Display the pinned nodes.")

display.predefineLayer('Skeleton', '<topmost>', pinnedNodesDisplay())
