// -*- C++ -*-
// $RCSfile: nodalequation.h,v $
// $Revision: 1.5 $
// $Author: reida $
// $Date: 2003/04/02 14:52:27 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>
#include <iostream>

// NodalEquations represent one component of one Equation at a Node.
// They are to the rows of the stiffness matrix what the
// DegreeOfFreedoms are to the columns.


typedef unsigned char EQNFLAG;
static const EQNFLAG DEPENDENT = 1;

class NodalEquation {
private:
  NodalEquation(int n) : index_(n), flags(0), value_(0.0) {}
  ~NodalEquation() {}
  friend class FEMesh;	// only a  FEMesh can create or destroy a NodalEquation

  int index_;
  double value_;
  EQNFLAG flags;
public:
  int index() const { return index_; }

  double value() const { return value_; } // Same value
  double &value() { return value_; }      // structure as DOF's.

  // A "dependent" nodal equation is an equation whose force (rhs) is
  // unknown, and which can be computed *after* the values of all the
  // degrees of freedom are found.  A dependent equation is not solved
  // during the solution step.
  void dependent() { flags |= DEPENDENT; }
  void independent() { flags &= ~DEPENDENT; }
  bool is_dependent() const { return flags & DEPENDENT; }
};

std::ostream &operator<<(std::ostream &os, const NodalEquation &neqn);
