# -*- python -*-
# $RCSfile: pixelselect.py,v $
# $Revision: 1.12 $
# $Author: langer $
# $Date: 2004/05/07 18:57:41 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from SWIG.engine import pixelselectioncouriere
from common import pixelselection
from common import pixelselectionmod
from common import registeredclass
from common.IO import whoville
from engine import skeletoncontext

ElementSelection = pixelselectioncouriere.ElementSelection
class SelectPixelsInElement(pixelselectionmod.SelectionModifier):
    def __init__(self, skeleton):
        self.skeleton = skeleton
    def __call__(self, ms, selection):
        skel = skeletoncontext.skeletonContexts[self.skeleton].getObject()
        selection.start()
        for el in skel.elements:
            if el.selected:
                selection.select(ElementSelection(ms, el))

registeredclass.Registration('Select Element Pixels',
                             registeredclass=pixelselectionmod.SelectionModifier,
                             subclass=SelectPixelsInElement,
                             ordering=100,
                             params=[
    whoville.WhoParameter('skeleton', skeletoncontext.skeletonContexts)
    ],
                             tip="Select the pixels underlying the selected elements.")

SegmentSelection = pixelselectioncouriere.SegmentSelection
class SelectPixelsUnderSegment(pixelselectionmod.SelectionModifier):
    def __init__(self, skeleton):
        self.skeleton = skeleton
    def __call__(self, ms, selection):
        skel = skeletoncontext.skeletonContexts[self.skeleton].getObject()
        selection.start()        
        for segment in skel.segments.values():
            if segment.selected:
                n0 = segment.nodes()[0].position()
                n1 = segment.nodes()[1].position()
                selection.select(SegmentSelection(ms, n0, n1))

registeredclass.Registration('Select Segment Pixels',
                             registeredclass=pixelselectionmod.SelectionModifier,
                             subclass=SelectPixelsUnderSegment,
                             ordering=105,
                             params=[
    whoville.WhoParameter('skeleton', skeletoncontext.skeletonContexts)
    ],
                             tip="Select the pixels underlying the selected skeleton edges")
                             
