// -*- C++ -*-
// $RCSfile: skelstiffness.C,v $
// $Revision: 1.2 $
// $Author: redwing $
// $Date: 2004/09/20 20:58:10 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>
#include "engine/field.h"
#include "engine/flux.h"
#include "engine/material.h"
#include "engine/property/elasticity/skelstiffness/skelstiffness.h"
#include "engine/element.h"
#include "engine/matvec.h"

using namespace std;

// Coef's.  The idea is that the shear modulus is going to be 
// mu0 + alpha*(T-t0), i.e. varies with slope alpha relative to 
// reference temperature t0.  In principle, could be arbitrarily
// complex, of course.
SkeletonStiffness::SkeletonStiffness(PyObject *registry,
				     const string &nm, 
				     Cijkl *cijkl)
  : Elasticity(nm, registry),
    cijkl_(*cijkl)
{
  temperature = dynamic_cast<ScalarField*>(Field::getField("Temperature"));
}

const Cijkl SkeletonStiffness::cijkl(const FEMesh *m, const Element *e, 
				    const MasterPosition &x) const {
  // Get local temperature
  const OutputValue tfield = e->outputField(m, *temperature, x);
  const ScalarOutputVal *tval =
    dynamic_cast<const ScalarOutputVal*>(tfield.valuePtr());
  double temp = tval->value();

  Cijkl modulus;
  // Because temperature variation of modulus is defined in terms
  // of lambda and mu, we have to convert to that representation.

  double lambda = cijkl_(0,0);
  double mu = (cijkl_(0,0)-cijkl_(0,1))/2.0;

  modulus.clear();
  // Construct temperature-dependent modulus from lambda and mu.
  modulus(0,0) = modulus(1,1) = modulus(2,2) = lambda+2.0*mu;
  modulus(0,1) = modulus(0,2) = modulus(1,2) = lambda;
  modulus(3,3) = modulus(4,4) = modulus(5,5) = mu;
  // 
  return modulus;
}
