# -*- python -*-
# $RCSfile: postprocess.py,v $
# $Revision: 1.9 $
# $Author: langer $
# $Date: 2004/09/18 21:47:40 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

import oofpaths

from tutorials import tutorial
TutoringItem = tutorial.TutoringItem
TutorialClass = tutorial.TutorialClass

      
TutorialClass(
    subject = "Postprocessing",
    ordering = 4,
    lessons = [
      
    TutoringItem(
    subject="Introduction",
    comments=

    """OOF2 provides various post-processing tools for viewing and
    analyzing a solution.  This tutorial provides a brief discussion
    of these tools."""  ),

    TutoringItem(
    subject="Example Mesh",
    comments=

    """Let us load an example Mesh from BOLD(File) menu.  In the
    file selector, select or type in BOLD(%s/cyallow.mesh).  Click
    BOLD(OK) to load the mesh.""" % oofpaths.examples,
    signal = ("new who" ,"Mesh")
    ),

        TutoringItem(
    subject="Graphics window",
    comments=

    """Open a graphics window to display the mesh.  If you've completed
    the tutorial BOLD(Quick Tour), you should be familiar with this
    mesh.
    
    First, we'll go over standard features such as displaying contour
    (filled or line) plots and element/node query."""
    ),

        TutoringItem(
    subject="New layer",
    comments=

    """Let us display a filled contour of the displacement
    in the x-direction, which is one of few non-trivial results
    - the mesh is subjected to an uniaxial tension in the x-direction.
    
    In the graphics window, create a new layer from BOLD(New) of
    BOLD(Layer) menu of the graphics window.  A layer editor will
    pop up.  Select BOLD(Mesh) for BOLD(Category).  If you have other
    meshes in the system, make sure you choose BOLD(cyallow.png)
    from the second option menu that represents the microstructure
    of the mesh.
    
    Move to the next slide for further instruction."""
    ),
        
        TutoringItem(
    subject="Display method",
    comments=

    """Now, move to BOLD(Display Methods) pane and you will notice
    that there is none present at the moment.Click BOLD(New).  A display
    method editor will pop up.  Click on the topmost button to list
    all the available display methods for the mesh.  Select
    BOLD(FilledContour).
    
    In the BOLD(what) option block, select BOLD(Field) and
    BOLD(Component).  Then, select BOLD(Displacement) and BOLD(x)
    for the field and the component, respectively.
        
    The option BOLD(where) determines whether to plot the contour at
    the displaced or original position.  Select BOLD(original).
    
    Leave the next five options as they are.  Click BOLD(OK) to display
    the contour, which will send you back to the layer editor."""
    ),
        
        TutoringItem(
    subject="More levels for contour",
    comments=
    """Bring up the graphics window to check the contour displayed.  If
    you want to put more levels in the contour, go back to
    the layer editor.  Make sure to select BOLD(FilledContour) from
    the BOLD(Display Methods) pane and click BOLD(Edit...).
    
    Look for the parameter BOLD(levels) and type in BOLD(21).  Click
    BOLD(OK).  The contour plot in the graphics window should reflect
    the change accordingly."""
    ),
        
        TutoringItem(
    subject="Values for the level",
    comments=

    """The actual value range for the color can be identified
    in the BOLD(Contour Map) pane of the graphics window.

    For example, if you click on anywhere on the contour map, the value range
    for the clicked color-block should be displayed in the bottom
    of the contour map pane.
    """
    ),
        
        TutoringItem(
    subject="Mesh info toolbox",
    comments=

    """While contour plots are effective in presenting the overall
    trend, if you are interested in result data at specific locations,
    you need to use other tools.  OOF2 provides BOLD(Mesh Info) toolbox
    for this kind of situation.
    
    Open BOLD(Mesh Info) toolbox from the graphics window.  The toolbox
    lets you query complete information on elements and nodes"""
    ),
        
        TutoringItem(
    subject="Querying node data",
    comments=

    """The first order of business in using the toolbox is to set
    the mode.  Set the toolbox in the node mode by clicking on the label
    BOLD(Node) or the check box for it.
    
    Click on any node of your interest.  The queried node is highlighted
    as a blue dot and its basic informations such as index, position are
    displayed in the information frame.

    To display associated field data (flux data is not available),
    displacement in this case, click on the check box for
    BOLD(Displacement).""",
    signal = "redraw"
    ),
        
        TutoringItem(
    subject="Querying element data",
    comments=

    """Now, switch the toolbox to the element mode.  As soon as you
    switch the mode, you will notice an element is automatically
    selected.  It's the closest element to the last mouse point from
    the previous mode.
    
    By default the basics including index, nodes,
    material are displayed.  To query BOLD(Displacement) and BOLD(Stress)
    data, click on the corresponding check boxes from the BOLD(Fields)
    and BOLD(Fluxes) frame, respectively.  It is a good idea to expand
    the graphics window to have a better access to the displayed
    values.
    
    The queried values are calculated at the mouse point.  Thus, if you
    click on a different point in the same element, values will be
    refreshed accordingly.""",
    signal = "redraw"
    ),
        
        TutoringItem(
    subject="Clearing up",
    comments=

    """There are three utility buttons at the bottom of
    the toolbox.  Their usage should be self-evident.  Click on
    the BOLD(Clear) button, which will effectively remove all the data
    from the toolbox and the element selection from the canvas.""",
    signal = "redraw"
    ),
        
        TutoringItem(
    subject="Mesh cross section",
    comments=

    """Imagine a situation where you want to extract field or flux
    data along the specified path.  BOLD(Mesh Cross Section) toolbox
    lets you export data calculated along the user-created path.
    
    Open the toolbox.  The BOLD(Source) frame shows which data from
    which mesh is to be processed.  For this to
    properly work, a graphics window should contain at least one
    contour layer, topmost or not.
    
    A path is created by click-and-drag of the mouse.  At this point,
    only a straight path can be created. Created paths are automatically
    added to the pool and they can be managed through buttons in
    BOLD(Cross Section) frame.
    
    Click BOLD(Next) for the further instruction.""",
    signal = "redraw"
    ),
        
        TutoringItem(
    subject="Mesh cross section - Continued",
    comments=

    """Create a path - it will be displayed as a gray line on
    on the canvas.  And rename it to BOLD(MyCrossSection).  No. of sampling
    points along the path can also be adjusted.
    
    The destination of the extracted data can be either the message
    window or a file.  To create a file for the data, select
    BOLD(New...) from the BOLD(Destination) menu. Select
    BOLD(Message Window) for the destination and give it a BOLD(Go!)""",
    signal = "redraw"
    ),

        TutoringItem(
    subject="Mesh cross section - Final",
    comments=

    """Bring up the message window.  You will see a bunch of numbers
    have been dumped here.  Each line represents a point and its
    corresponding value - BOLD(x), BOLD(y), and BOLD(Displacement x)."""
    ),
        
        TutoringItem(
    subject="Analysis page",
    comments=

    """Move to BOLD(Analysis) page.  While BOLD(Mesh Cross Section)
    toolbox only works for results
    that are present in the graphics window, BOLD(Analysis) page doesn't
    require results that are being processed to be present in the graphics
    window.  Also, BOLD(Analysis) pages features more output operations. Click
    on the option menu bar in the BOLD(Operation) pane to see all
    the available options.  The source data can be selected in the BOLD(Data)
    pane."""
    ),

        TutoringItem(
    subject="Analysis page - Grid",
    comments=

    """Let us try BOLD(Grid) operation.  Select BOLD(Grid) from
    the BOLD(Operation) pane and set number of points for both axes
    to be BOLD(5).  This will create a uniform grid of 25 points covering
    the entire mesh.  The selected data item will be calculated at each
    of the grid point.

    As for the data item, select BOLD(Field) and BOLD(Derivative).  As soon
    as you select BOLD(Derivative), another menu bar should appear.  Select
    BOLD(Component).  So far, we've decided that we're going to output
    a derivative of a field component.  Next three option menus are for
    specific choices.  Select  BOLD(Displacement), BOLD(x), and BOLD(x)
    in order - it's a normal strain in the BOLD(x)-direction.

    Set the controls for the heart of the message window and
    give it a BOLD(Go).  Check the output  in the message window."""
    ),
    
        TutoringItem(
    subject="Analysis page - Final",
    comments=

    """Other available operations for processing data include BOLD(Average),
    BOLD(Standard Deviation), and BOLD(Cross Section).

    BOLD(Average) and BOLD(Standard Deviation) are computed from
    all elements in the mesh.  The parameter BOLD(order) controls
    the order of integration over the element.

    The last option BOLD(Cross Section) does what BOLD(Mesh Cross Section)
    toolbox does.  The cross sections (paths) from the toolbox are
    also accessible from here.  The difference between the two is that
    here, you have more control over what to output."""
    ),

        TutoringItem(
    subject="Boundary analysis",
    comments=

    """Move to BOLD(Boundary Analysis) page, which is the final
    subject for this tutorial.  This page does the simple
    thing - outputting the integrated sum of the flux along
    the specified boundary.

    Select BOLD(right) from the BOLD(boundary) pull-down menu. And
    BOLD(Stress) for BOLD(flux).  Set the destination to
    BOLD(Message Window) and click BOLD(Go!) button.  A list of two
    values are reported.  In this example, they are the sum of reaction
    forces in the x and y-direction along the right side of the mesh.

    You've just finished the last of the tutorial.  Thanks for trying them
    out."""
    )
    
    ])
